;---------------------------------------------------------------------------
; Chorus Effects
;---------------------------------------------------------------------------

sr=44100
kr=22050
ksmps=2
nchnls=2
zakinit 30, 30

;---------------------------------------------------------------------------
; Pluck Physical Model
;---------------------------------------------------------------------------
       instr  2

iamp   =      p4          ; Amplitude
ifqc   =      cpspch(p5)  ; Convert to frequency
itab1  =      p6          ; Initial table
imeth  =      p7          ; Decay method
ioutch =      p8          ; Output channel

aplk   pluck  iamp, ifqc, ifqc, itab1, imeth       ; Pluck waveguide model
       zawm   aplk, ioutch                           ; Write to output
       endin

;---------------------------------------------------------------------------
; LFO
;---------------------------------------------------------------------------
          instr   7
iamp      init    p4
ifqc      init    p5
itab1     init    p6
iphase    init    p7
ioffset   init    p8
iout      init    p9

koscil    oscil   iamp, ifqc, itab1, iphase  ;Table oscillator
kout      =       koscil+ioffset

          zkw     kout, iout           ;Send to output channel

          endin

;---------------------------------------------------------------------------
;Lorenz Attractor
;---------------------------------------------------------------------------
        instr  8

iamp    init   p4/40
kx      init   p5
ky      init   p6
kz      init   p7
is      init   p8
ir      init   p9
ib      init   p10
ih      init   p11/10000
ioutch  init   p12
ioffset init   p13

kxnew   =      kx+ih*is*(ky-kx)
kynew   =      ky+ih*(-kx*kz+ir*kx-ky)
kznew   =      kz+ih*(kx*ky-ib*kz)

kx      =      kxnew
ky      =      kynew
kz      =      kznew

printk   .5, kx
printk   .5, ky
printk   .5, kz

        zkw    kx*iamp+ioffset, ioutch
        endin

;---------------------------------------------------------------------------
; Chorus
;---------------------------------------------------------------------------
         instr   35

imixch   =       p4             ; Mix of chorused signal
imix     =       1-imixch       ; Mix of direct signal
izin     =       p5            ; Input channel
izout    =       p6            ; Output channel
ikin     =       p7            ; Input K Channel

asig     zar     izin           ; Read input channel
kinsig   zkr     ikin           ; Read K Input

adel1    vdelay  asig, kinsig, 100      ; Variable delay tap
aout     =       adel1*imixch+asig*imix ; Mix direct and chorused signals

;aout     =       kinsig*500
         zaw     aout, izout                    ; Write to output channel

         endin

;---------------------------------------------------------------------------
; Mixer Section
;---------------------------------------------------------------------------
          instr 100

asig1     zar   p4             ; Read input channel 1
igl1      init  p5*sqrt(p6)    ; Left gain
igr1      init  p5*sqrt(1-p6)  ; Right gain

kdclick   linseg  0, .001, 1, p3-.002, 1, .001, 0  ; Declick

asigl     =     asig1*igl1       ; Scale and sum
asigr     =     asig1*igr1

          outs  kdclick*asigl, kdclick*asigr   ; Output stereo pair

          endin

;---------------------------------------------------------------------------
; Clear audio & control channels
;---------------------------------------------------------------------------
          instr 110

          zacl  0, 30          ; Clear audio channels 0 to 30
          zkcl  0, 30          ; Clear control channels 0 to 30

          endin

