# Generate Score
$scofile="auto.sco";      # Name of score file
open (SCO, ">$scofile");  # Open file for output
                          # Print an additive table
print SCO "f1 0 65536 10 1\n";
print SCO "f2 0 1024 10 1 .3 .1 0 .2 .02 0 .1 .04\n";

#     Sta  Dur  Fqc  FqcRng  Grains
#grain(0,   4,   900, .21,    200);
#grain(4,   2,   300, .10,    100);
#grain(6,   3,   330, .10,    100);
#grain(7,   3,   600, .021,   150);
#grain(9,   2,   920, .03,    180);
#grain(11,  3,   330, .12,    110);
#grain(12,  4,   660, .014,   150);
#grain(13,  2,  2200, .034,   150);

#     Sta  Dur  Fqc  XCorner YCorner Step
#circsq(0,  3,   220, -5.532,  -3.635,   2.1572);
#circsq(6,  3,   440, -5.232,  -3.135,   2.7572);

ffract();

close (SCO);             # Close the score file

# Grain subroutine for creating a granular texture.
sub grain                   # grain subroutine
 {
  my $strt    = $_[0];      # local variable $strt is param 0
  my $dur     = $_[1];      # $dur     is set to param list item 1
  my $freq    = $_[2];      # $freq    is set to param list item 2
  my $frange  = $_[3];      # $frange  is set to param list item 3
  my $ngrains = $_[4];      # $ngrains is set to param list item 4

  for ( $i=0; $i<$ngrains; $i++)  # for each grain do
   {
    $rnd1 = rand(1);              # random number 0-1
    $sta  = $rnd1*$dur+$strt;     # grain start time somewhere
                                  # around duration of note + offset
    $dur1 = rand($dur)+.01;       # grain duration
    $amp  = 1000;                 # amplitude
    $fqc  = rand($frange*$freq)+$freq;  # frequency is a random number
                                  # in the frequency rage
    $pan  = rand(1);              # pan each grain randomly
    print SCO "i1 $sta $dur1 $amp $fqc $pan\n"; # print to the score file
   }
 }

sub circsq
 {
  my $sta     = $_[0];
  my $dur     = $_[1];
  my $fqc     = $_[2];
  my $xcorner = $_[3];
  my $ycorner = $_[4];
  my $step    = $_[5];
  $nsteps     = 10;
  $numfqc     = 20;
  $amp        = 2000;

  $sta1       = $sta;
  $x=$xcorner;
  for ( $i=0; $i<$nsteps; $i++)
   {
    $y=$ycorner;
    for ( $j=0; $j<$nsteps; $j++)
     {
      $z    = ($x*$x+$y*$y)%$numfqc;
      $fqc1 = $z*$fqc/10+$fqc;
      $pan  = rand(1);
      print SCO "i1 $sta $dur $amp $fqc1 $pan\n";
      $y+=$step;
      $sta=rand($dur)+.02;
     }
    $x+=$step;
   }
 }

# Feather Fractal
sub ffract
 {
  $nsteps = 100;
  $h   = .001;
  $a   = -.48;
  $c   = 2-2*$a;
  $b   = .93;
  $x   = 3;
  $y   = 0;
  $w   = $a*3+$c*.9;
  $amp = 5000;

  for ( $i=0; $i<$nsteps; $i++)
   {
    $z   = $x;
    $x   = $b*$y+$w;
    $u   = $x*$x;
    $w   = $a*$x+$c*$u/(1+$u);
    $y   = $w-$z;
    $sta = (10+$z)/2;
    $dur = (10+$y)/4;
    $fqc = 440+$x*12;
    $pan = rand(1);
    $fco = 2000+$w*10;
    $rez = (10+$x);
    print SCO "i3 $sta $dur $amp $fqc $pan $fco $rez\n";
   }
 }
