;---------------------------------------------------------------
; A set of analog modeling instruments using resonant low-pass
; filter by Hans Mikelson November 1998
;---------------------------------------------------------------
sr = 44100
kr = 22050
ksmps = 2
nchnls = 2
zakinit 50, 50

;---------------------------------------------------------------
; LFO
;---------------------------------------------------------------
           instr   5

idur       =       p3
iamp       =       p4
ifqc       =       p5
iwave      =       p6
ioffset    =       p7
ioutch     =       p8

ksig       oscil   iamp, ifqc, iwave
kout       =       ksig*.5+ioffset
           zkw     kout, ioutch

           endin

;---------------------------------------------------------------
; VCO with Rezzy Filter
;---------------------------------------------------------------
           instr   11

idur       =       p3              ; Duration
iamp       =       p4              ; Amplitude
ifqc       =       cpspch(p5)      ; Frequency
irtfqc     =       sqrt(ifqc)      ; Square root of frequency
ifco       =       p6*ifqc/500     ; Filter cut off (higher frequencies get a higher cut off)
krez       init    p7              ; Resonance does not change in this instrument
iwave      =       p8              ; Selected wave form 1=Saw, 2=Square/PWM, 3=Tri/Saw-Ramp-Mod
ipwch      =       p9              ; PWM Channel
ipbend     =       p10             ; Pitch Bend Table
isine      =       1
imaxd      =       2*ifqc

ileak      =       (iwave==2 ? .999 : (iwave==3 ? .995 : .999)) ; Leaky integration value

kpw        zkr     ipwch           ; Read the Zak modulation channel
kpwn       =       kpw-.5          ; Get a pulse width that goes from -.5 to .5
kfco       expseg  100+.5*ifco, .2*idur, ifco+100, .5*idur, ifco*.8+100, .1*idur, .5*ifco+100 ; Filter envelope
kpbend     oscil   1, 1/idur, ipbend  ; Pitch bend
kamp       linseg  0, .01*idur, iamp, .2*idur, .9*iamp, .59*idur, .9*iamp, .2*idur, 0            ; Amp envelope
aamp       interp  kamp
kfqc       =       ifqc*kpbend

asig       vco     1, kfqc, iwave, kpw, isine, imaxd

aout       rezzy   asig, kfco*kpbend, krez                      ; Apply the filter

           outs    aout*aamp, aout*aamp                         ; Stereo ouput and amplification
           endin

;---------------------------------------------------------------
; Moog VCO with Mikleson filter
;---------------------------------------------------------------
           instr   12

idur       =       p3              ; Duration
iamp       =       p4              ; Amplitude
ifqc       =       cpspch(p5)      ; Frequency
irtfqc     =       sqrt(ifqc)      ; Square root of frequency
ifco       =       p6*ifqc/500     ; Filter cut off (higher frequencies get a higher cut off)
krez       init    p7              ; Resonance does not change in this instrument
iwave      =       p8              ; Selected wave form 1=Saw, 2=Square/PWM, 3=Tri/Saw-Ramp-Mod
ipwch      =       p9              ; PWM Channel
ipbend     =       p10             ; Pitch Bend Table
isine      =       1
imaxd      =       2*ifqc

ileak      =       (iwave==2 ? .999 : (iwave==3 ? .995 : .999)) ; Leaky integration value

kpw        zkr     ipwch           ; Read the Zak modulation channel
kpwn       =       kpw-.5          ; Get a pulse width that goes from -.5 to .5
kfco       expseg  100+.1*ifco, .2*idur, ifco+100, .5*idur, ifco*.2+100, .1*idur, .4*ifco+100 ; Filter envelope
kpbend     oscil   1, 1/idur, ipbend  ; Pitch bend
kamp       linseg  0, .01*idur, iamp, .2*idur, .8*iamp, .59*idur, .5*iamp, .2*idur, 0            ; Amp envelope
aamp       interp  kamp
kfqc       =       ifqc*kpbend

asig       vco     1, kfqc, iwave, kpw, isine, imaxd

aout       rezzy   asig, kfco*kpbend, krez                      ; Apply the filter

           outs    aout*aamp, aout*aamp                         ; Stereo ouput and amplification
           endin

;---------------------------------------------------------------
; Moog VCO with Mikleson filter
;---------------------------------------------------------------
           instr   13

idur       =       p3              ; Duration
iamp       =       p4              ; Amplitude
ifqc       =       cpspch(p5)      ; Frequency
irtfqc     =       sqrt(ifqc)      ; Square root of frequency
ifco       =       p6*ifqc/500     ; Filter cut off (higher frequencies get a higher cut off)
krez       init    p7              ; Resonance does not change in this instrument
iwave      =       p8              ; Selected wave form 1=Saw, 2=Square/PWM, 3=Tri/Saw-Ramp-Mod
isine      =       1
imaxd      =       2/ifqc

kpw        init    .5           ; Read the Zak modulation channel
kfco       expseg  100+.1*ifco, .2*idur, ifco, .6*idur, ifco, .2*idur, .4*ifco+100     ; Filter envelope
kamp       linseg  0, .2*idur, iamp, .6*idur, iamp, .2*idur, 0  ; Amp envelope
aamp       interp  kamp
kvibr      oscil   .002, 6, 1
kfqc       =       ifqc*(1 + kvibr)
kfclf1     oscil   .5, .5, 1
kfclf2     oscil   .5, .6, 1

asig1      vco     1, kfqc*.998,  iwave, kpw, isine, imaxd
asig2      vco     1, kfqc*.999,  iwave, kpw, isine, imaxd
asig3      vco     1, kfqc*1.001, iwave, kpw, isine, imaxd
asig4      vco     1, kfqc*1.002, iwave, kpw, isine, imaxd

aoutl1     rezzy   asig1+asig3, kfco*0.99*(1+kfclf1), krez          ; Apply the filter
aoutr1     rezzy   asig2+asig4, kfco*1.01*(1+kfclf2), krez          ; Apply the filter
aoutl2     rezzy   asig1+asig3, kfco*1.51*(1+kfclf1), krez          ; Apply the filter
aoutr2     rezzy   asig2+asig4, kfco*1.49*(1+kfclf2), krez          ; Apply the filter

           outs    (aoutl1+aoutl2)*aamp, (aoutr1+aoutr2)*aamp       ; Stereo ouput and amplification
           endin

